--2781/2/4
CREATE DATABASE TestTriggerAudit

USE TestTriggerAudit

CREATE TABLE DataTable
(
	ID INT IDENTITY PRIMARY KEY,
	Name NVARCHAR(50),
	ValidTo DATETIME)
 
CREATE TABLE DataTable_log
(
	AuditLogId INT IDENTITY PRIMARY KEY,
	ID INT,
	Name NVARCHAR(50),
	ValidTo DATETIME,
	ModificationDate DATETIME,
	ModifiedBy SYSNAME,
	Action CHAR(1)
) 
GO

CREATE TRIGGER DataTable_Insert ON DataTable
FOR INSERT
AS
	INSERT DataTable_log 
		SELECT i.*,GETDATE(),USER_NAME(),'I' FROM inserted i
GO

CREATE TRIGGER DataTable_Update ON DataTable
FOR UPDATE
AS
	INSERT DataTable_log 
		SELECT i.*,GETDATE(),USER_NAME(),'U' FROM inserted i
GO

CREATE TRIGGER DataTable_Delete ON DataTable
FOR DELETE
AS
	INSERT DataTable_log 
		SELECT d.*,GETDATE(),USER_NAME(),'D' FROM deleted d
GO

INSERT DataTable VALUES ('Nazwa 1', GETDATE())
UPDATE DataTable SET Name='Nazwa 2 bis' WHERE ID=1
DELETE FROM DataTable WHERE ID=1

SELECT * FROM DataTable
SELECT * FROM DataTable_log

--2781/2/5
CREATE DATABASE TestSchemas

USE TestSchemas
GO
CREATE SCHEMA Production
GO
CREATE SCHEMA Marketing
GO
CREATE USER d1 WITHOUT LOGIN WITH DEFAULT_SCHEMA=Production
CREATE USER d2 WITHOUT LOGIN WITH DEFAULT_SCHEMA=Marketing
EXEC sp_addrolemember 'db_ddladmin', 'd1'
EXEC sp_addrolemember 'db_ddladmin', 'd2'

GRANT SELECT ON SCHEMA::Production TO d1,d2
GRANT SELECT ON SCHEMA::Marketing TO d1,d2


EXECUTE AS USER='d1'
	CREATE TABLE Product
	(Id INT IDENTITY,
	 ProductProductionName  NVARCHAR(50))
	SELECT * FROM Product
REVERT

EXECUTE AS USER='d2'
	CREATE TABLE Product
	(Id INT IDENTITY,
	 ProductMarketingName  NVARCHAR(50))
	SELECT * FROM Product
REVERT

CREATE USER u1 WITHOUT LOGIN WITH DEFAULT_SCHEMA=Production
CREATE USER u2 WITHOUT LOGIN 
GRANT SELECT ON SCHEMA::Production TO u1,u2
GRANT SELECT ON SCHEMA::Marketing TO u1,u2

EXECUTE AS USER='u1'
	SELECT * FROM Production.Product
	SELECT * FROM Product
REVERT

EXECUTE AS USER='u2'
	SELECT * FROM Production.Product
	SELECT * FROM Product
REVERT


--trigger DDL
USE AdventureWorks2008R2
go
CREATE TRIGGER safety
ON DATABASE
FOR DROP_TABLE, ALTER_TABLE
AS
	PRINT 'You must disable Trigger "safety" to drop or alter tables!'
	ROLLBACK

DROP TABLE Sales.SalesOrderDetail

DROP TRIGGER safety ON DATABASE




CREATE TRIGGER TR_DATABASE_DDL_TRACKING
ON DATABASE FOR DDL_DATABASE_LEVEL_EVENTS
AS
  DECLARE @PostTime datetime2 = EVENTDATA().value('(/EVENT_INSTANCE/PostTime)[1]','datetime2');
  DECLARE @LoginName sysname = EVENTDATA().value('(/EVENT_INSTANCE/LoginName)[1]','sysname');
  DECLARE @TSQLCommand nvarchar(max) = EVENTDATA().value('(/EVENT_INSTANCE/TSQLCommand/CommandText)[1]','nvarchar(max)');
  
  PRINT 'DDL Event Occurred';
  PRINT @LoginName;
  PRINT 'executed';
  PRINT @TSQLCommand;
  PRINT 'at';
  PRINT @PostTime;
GO


CREATE TABLE TestTable (TestTableID int);
GO
DROP TABLE TestTable;
GO


DROP TRIGGER TR_DATABASE_DDL_TRACKING ON DATABASE;
GO



CREATE TRIGGER TR_DDL_ProcNamingConvention
ON DATABASE
FOR CREATE_PROCEDURE
AS 
BEGIN
  SET NOCOUNT ON;

  DECLARE @EventData xml;
  DECLARE @ObjectName sysname;
	
  SET @EventData = EVENTDATA();
  SET @ObjectName = @EventData.value('(/EVENT_INSTANCE/ObjectName)[1]','sysname');
	
  IF @ObjectName LIKE 'sp_%'
  BEGIN
    PRINT 'Stored Procedure names are not permitted to start with sp_';
    ROLLBACK TRAN;
  END;
END;
GO


CREATE PROC GetVersion
AS
  SELECT @@VERSION;
GO

CREATE PROC sp_GetVersion
AS
  SELECT @@VERSION;
GO

DROP PROC GetVersion;
GO

CREATE TRIGGER ddl_trig_login
ON ALL SERVER
FOR DDL_LOGIN_EVENTS
AS
PRINT 'Login Event Issued.'
SELECT EVENTDATA().value('(/EVENT_INSTANCE/TSQLCommand/
CommandText)[1]','nvarchar(max)')


CREATE DATABASE TestXMLColumn

USE TestXMLColumn

CREATE TABLE TextXML
(ID INT IDENTITY PRIMARY KEY,
 x XML)
 
 INSERT TextXML VALUES(N'<packets><packet id="101">Yellow</packet><packet id="102">Black</packet></packets>')
 INSERT TextXML VALUES(N'<packets><packet id="201">Yellow</packet><packet id="202">Black</packet></packets>')
 INSERT TextXML VALUES(N'<packets><packet id="303">Yellow</packet></packets>')
 
SELECT id,x.value('(/packets/packet/@id)[1]','int')
FROM TextXML 

ALTER TABLE TextXML
ADD FirstValue AS x.value('(/packets/packet/@id)[1]','int')
GO

CREATE FUNCTION GetFirstValue(@x xml)
RETURNS INT
WITH SCHEMABINDING
AS
BEGIN
 RETURN @x.value('(/packets/packet/@id)[1]','int')
END 
GO

ALTER TABLE TextXML
ADD FirstValue AS dbo.GetFirstValue(x)

CREATE NONCLUSTERED INDEX ix_xml_first_value ON dbo.TextXML(FirstValue)

SELECT * FROM TextXML

INSERT TextXML VALUES(N'<packets><packet id="4">Yellow</packet></packets>')

SELECT id
FROM TextXML
WHERE x.exist( '/packets/packet[@id=sql:column("id")]') = 1
GO



--2781/3/1
--indeks xml

USE tempdb;
GO

-- Step 2 - Create a primary XML index
CREATE TABLE dbo.ProductImport
( ProductImportID int IDENTITY(1,1) PRIMARY KEY,
  ProductDetails xml (CONTENT dbo.ProductDetailsSchema)
);
GO


INSERT INTO dbo.ProductImport 
  (ProductDetails)
  VALUES ('
<Production.Product xmlns="urn:schemas-microsoft-com:sql:SqlRowSet3" ProductID="1" Name="Adjustable Race" ListPrice="0.0000" />
<Production.Product xmlns="urn:schemas-microsoft-com:sql:SqlRowSet3" ProductID="2" Name="Bearing Ball" ListPrice="0.0000" />
');
GO


CREATE PRIMARY XML INDEX IX_ProductImport_ProductDetails
ON dbo.ProductImport (ProductDetails);
GO

CREATE XML INDEX IX_ProductImport_ProductDetails_Value
ON dbo.ProductImport (ProductDetails)
USING XML INDEX IX_ProductImport_ProductDetails
FOR VALUE;
GO

SELECT * FROM sys.xml_indexes;
GO

DROP TABLE dbo.ProductImport;
GO

--modyfikacja XML
UPDATE TextXml
SET x.modify('
insert <packet id="333">Green</packet> 
before (/packets/packet[@id=101])[1]')
WHERE Id=1

UPDATE TextXml
SET x.modify('
insert <packet id="444">Green</packet> 
as first into (/packets)[1]')
WHERE Id=1

SELECT * FROM TextXML

--2781/3/2 indeksy - chwilowe wylaczanie

SELECT * FROM sys.indexes
WHERE object_id = OBJECT_ID('HumanResources.Shift')

ALTER INDEX AK_Shift_Name ON HumanResources.Shift DISABLE

SELECT name, is_disabled FROM sys.indexes
WHERE object_id = OBJECT_ID('HumanResources.Shift')

ALTER INDEX AK_Shift_Name ON HumanResources.Shift REBUILD

ALTER INDEX AK_Shift_Name ON HumanResources.Shift REBUILD
WITH (online=on, sort_in_tempdb=on)

ALTER INDEX ALL ON HumanResources.Shift DISABLE

ALTER INDEX ALL ON HumanResources.Shift REBUILD


--2781/4/1,2
--Window 1
USE AdventureWOrks2008R2
SELECT ProductId, name, Color FROM Production.Product
BEGIN TRAN
	UPDATE Production.Product
	SET Color = 'x' WHERE ProductId=317
ROLLBACK

--Window 2
USE AdventureWOrks2008R2
SELECT ProductId, name, Color FROM Production.Product
BEGIN TRAN
	SELECT COUNT(*) FROM Production.Product
	WHERE COlor='Black'
ROLLBACK

--deadlock
--Window 1
USE AdventureWOrks2008R2
SELECT ProductId, name, Color FROM Production.Product
SELECT * FROM Production.ProductCategory
BEGIN TRAN
	UPDATE Production.Product
	SET Color = 'x' WHERE ProductId=317
	
	UPDATE Production.ProductCategory
	SET Name='x' WHERE ProductCategorId=1
ROLLBACK


--Window 2
USE AdventureWOrks2008R2
BEGIN TRAN
	UPDATE Production.ProductCategory
	SET Name='y' WHERE ProductCategorId=1
	
	UPDATE Production.Product
	SET Color = 'y' WHERE ProductId=317
ROLLBACK
	
--278/4/3	
--hinty	
SELECT p.Size, AVG(p.ListPrice) 
FROM Production.Product AS p WITH (TABLOCKX)
WHERE p.Color='Red'
GROUP BY p.Size


SELECT p.Size, AVG(p.ListPrice) 
FROM Production.Product AS p WITH (TABLOCKX, HOLDLOCK)
WHERE p.Color='Red'
GROUP BY p.Size


--window 1
BEGIN TRANSACTION UPDATE  PRODUCTION.Product
SET Name='1111'
WHERE ProductID=3
--window 2
SELECT *
FROM Production.Product AS p WITH (READPAST)
WHERE p.ProductID in (1,2,3,4)